@@ -1,64 +1,65 @@
-# Copyright (C) 2017-2019  The Software Heritage developers
+# Copyright (C) 2017-2020  The Software Heritage developers
 # See the AUTHORS file at the top-level directory of this distribution
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
 
-import hashlib
-
-from typing import Sequence, Type
-
 from abc import ABCMeta, abstractmethod
-from django.urls import reverse
-from django.http import HttpResponse
+import datetime
+import hashlib
+import json
+from typing import Any, Dict, Optional, Sequence, Tuple, Type, Union
+
+from django.http import FileResponse, HttpResponse
 from django.shortcuts import render
+from django.urls import reverse
 from django.utils import timezone
 from rest_framework import status
 from rest_framework.authentication import BaseAuthentication, BasicAuthentication
 from rest_framework.permissions import BasePermission, IsAuthenticated
+from rest_framework.request import Request
 from rest_framework.views import APIView
 
 from swh.model import hashutil
 from swh.scheduler.utils import create_oneshot_task_dict
 
 from ..config import (
-    SWHDefaultConfig,
+    ARCHIVE_KEY,
+    ARCHIVE_TYPE,
+    CONT_FILE_IRI,
+    DEPOSIT_STATUS_DEPOSITED,
+    DEPOSIT_STATUS_LOAD_SUCCESS,
+    DEPOSIT_STATUS_PARTIAL,
     EDIT_SE_IRI,
     EM_IRI,
-    CONT_FILE_IRI,
-    ARCHIVE_KEY,
     METADATA_KEY,
+    METADATA_TYPE,
     RAW_METADATA_KEY,
     STATE_IRI,
-    DEPOSIT_STATUS_DEPOSITED,
-    DEPOSIT_STATUS_PARTIAL,
-    DEPOSIT_STATUS_LOAD_SUCCESS,
-    ARCHIVE_TYPE,
-    METADATA_TYPE,
+    APIConfig,
 )
 from ..errors import (
-    MAX_UPLOAD_SIZE_EXCEEDED,
     BAD_REQUEST,
-    ERROR_CONTENT,
     CHECKSUM_MISMATCH,
-    make_error_dict,
-    MEDIATION_NOT_ALLOWED,
-    make_error_response_from_dict,
+    ERROR_CONTENT,
     FORBIDDEN,
-    NOT_FOUND,
-    make_error_response,
+    MAX_UPLOAD_SIZE_EXCEEDED,
+    MEDIATION_NOT_ALLOWED,
     METHOD_NOT_ALLOWED,
-    ParserError,
+    NOT_FOUND,
     PARSING_ERROR,
+    ParserError,
+    make_error_dict,
+    make_error_response,
+    make_error_response_from_dict,
 )
-from ..models import Deposit, DepositRequest, DepositCollection, DepositClient
+from ..models import Deposit, DepositClient, DepositCollection, DepositRequest
 from ..parsers import parse_xml
 
-
 ACCEPT_PACKAGINGS = ["http://purl.org/net/sword/package/SimpleZip"]
 ACCEPT_ARCHIVE_CONTENT_TYPES = ["application/zip", "application/x-tar"]
 
 
-class SWHAPIView(APIView):
+class AuthenticatedAPIView(APIView):
     """Mixin intended as a based API view to enforce the basic
        authentication check
 
@@ -68,12 +69,12 @@ class SWHAPIView(APIView):
     permission_classes: Sequence[Type[BasePermission]] = (IsAuthenticated,)
 
 
-class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
+class APIBase(APIConfig, AuthenticatedAPIView, metaclass=ABCMeta):
     """Base deposit request class sharing multiple common behaviors.
 
     """
 
-    def _read_headers(self, request):
+    def _read_headers(self, request: Request) -> Dict[str, Any]:
         """Read and unify the necessary headers from the request (those are
            not stored in the same location or not properly formatted).
 
@@ -113,6 +114,8 @@ class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
         on_behalf_of = meta.get("HTTP_ON_BEHALF_OF")
         metadata_relevant = meta.get("HTTP_METADATA_RELEVANT")
 
+        swhid = meta.get("HTTP_X_CHECK_SWHID")
+
         return {
             "content-type": content_type,
             "content-length": content_length,
@@ -123,9 +126,10 @@ class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
             "slug": slug,
             "on-behalf-of": on_behalf_of,
             "metadata-relevant": metadata_relevant,
+            "swhid": swhid,
         }
 
-    def _compute_md5(self, filehandler):
+    def _compute_md5(self, filehandler) -> bytes:
         """Compute uploaded file's md5 sum.
 
         Args:
@@ -142,31 +146,36 @@ class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
         return h.digest()
 
     def _deposit_put(
-        self, request, deposit_id=None, in_progress=False, external_id=None
-    ):
+        self,
+        request: Request,
+        deposit_id: Optional[int] = None,
+        in_progress: bool = False,
+        external_id: Optional[str] = None,
+    ) -> Deposit:
         """Save/Update a deposit in db.
 
         Args:
-            deposit_id (int): deposit identifier
-            in_progress (dict): The deposit's status
-            external_id (str): The external identifier to associate to
-              the deposit
+            request: request data
+            deposit_id: deposit identifier
+            in_progress: deposit status
+            external_id: external identifier to associate to the deposit
 
         Returns:
             The Deposit instance saved or updated.
 
         """
+        complete_date: Optional[datetime.datetime] = None
+        deposit_parent: Optional[Deposit] = None
+
         if in_progress is False:
             complete_date = timezone.now()
             status_type = DEPOSIT_STATUS_DEPOSITED
         else:
-            complete_date = None
             status_type = DEPOSIT_STATUS_PARTIAL
 
         if not deposit_id:
             try:
-                # find a deposit parent (same external id, status load
-                # to success)
+                # find a deposit parent (same external id, status load to success)
                 deposit_parent = (
                     Deposit.objects.filter(
                         external_id=external_id, status=DEPOSIT_STATUS_LOAD_SUCCESS
@@ -175,8 +184,10 @@ class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
                     .get()
                 )  # noqa
             except Deposit.DoesNotExist:
-                deposit_parent = None
+                # then no parent for that deposit, deposit_parent already None
+                pass
 
+            assert external_id is not None
             deposit = Deposit(
                 collection=self._collection,
                 external_id=external_id,
@@ -210,24 +221,24 @@ class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
 
     def _deposit_request_put(
         self,
-        deposit,
-        deposit_request_data,
-        replace_metadata=False,
-        replace_archives=False,
-    ):
+        deposit: Deposit,
+        deposit_request_data: Dict[str, Any],
+        replace_metadata: bool = False,
+        replace_archives: bool = False,
+    ) -> DepositRequest:
         """Save a deposit request with metadata attached to a deposit.
 
         Args:
-            deposit (Deposit): The deposit concerned by the request
-            deposit_request_data (dict): The dictionary with at most 2 deposit
-            request types (archive, metadata) to associate to the deposit
-            replace_metadata (bool): Flag defining if we add or update
+            deposit: The deposit concerned by the request
+            deposit_request_data: The dictionary with at most 2 deposit
+              request types (archive, metadata) to associate to the deposit
+            replace_metadata: Flag defining if we add or update
               existing metadata to the deposit
-            replace_archives (bool): Flag defining if we add or update
+            replace_archives: Flag defining if we add or update
               archives to existing deposit
 
         Returns:
-            None
+            the DepositRequest object stored in the backend
 
         """
         if replace_metadata:
@@ -247,7 +258,7 @@ class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
 
         metadata = deposit_request_data.get(METADATA_KEY)
         if metadata:
-            raw_metadata = deposit_request_data.get(RAW_METADATA_KEY)
+            raw_metadata = deposit_request_data[RAW_METADATA_KEY]
             deposit_request = DepositRequest(
                 type=METADATA_TYPE,
                 deposit=deposit,
@@ -257,27 +268,28 @@ class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
             deposit_request.save()
 
         assert deposit_request is not None
+        return deposit_request
 
-    def _delete_archives(self, collection_name, deposit_id):
-        """Delete archives reference from the deposit id.
+    def _delete_archives(self, collection_name: str, deposit_id: int) -> Dict:
+        """Delete archive references from the deposit id.
 
         """
         try:
             deposit = Deposit.objects.get(pk=deposit_id)
         except Deposit.DoesNotExist:
             return make_error_dict(
-                NOT_FOUND, "The deposit %s does not exist" % deposit_id
+                NOT_FOUND, f"The deposit {deposit_id} does not exist"
             )
         DepositRequest.objects.filter(deposit=deposit, type=ARCHIVE_TYPE).delete()
 
         return {}
 
-    def _delete_deposit(self, collection_name, deposit_id):
+    def _delete_deposit(self, collection_name: str, deposit_id: int) -> Dict:
         """Delete deposit reference.
 
         Args:
-            collection_name (str): Client's name
-            deposit_id (id): The deposit to delete
+            collection_name: Client's collection
+            deposit_id: The deposit to delete
 
         Returns
             Empty dict when ok.
@@ -288,7 +300,7 @@ class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
             deposit = Deposit.objects.get(pk=deposit_id)
         except Deposit.DoesNotExist:
             return make_error_dict(
-                NOT_FOUND, "The deposit %s does not exist" % deposit_id
+                NOT_FOUND, f"The deposit {deposit_id} does not exist"
             )
 
         if deposit.collection.name != collection_name:
@@ -306,27 +318,29 @@ class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
 
         return {}
 
-    def _check_preconditions_on(self, filehandler, md5sum, content_length=None):
+    def _check_preconditions_on(
+        self, filehandler, md5sum: str, content_length: Optional[int] = None
+    ) -> Optional[Dict]:
         """Check preconditions on provided file are respected. That is the
            length and/or the md5sum hash match the file's content.
 
         Args:
             filehandler (InMemoryUploadedFile): The file to check
-            md5sum (hex str): md5 hash expected from the file's content
-            content_length (int): the expected length if provided.
+            md5sum: md5 hash expected from the file's content
+            content_length: the expected length if provided.
 
         Returns:
             Either none if no error or a dictionary with a key error
             detailing the problem.
 
         """
+        max_upload_size = self.config["max_upload_size"]
         if content_length:
-            if content_length > self.config["max_upload_size"]:
+            if content_length > max_upload_size:
                 return make_error_dict(
                     MAX_UPLOAD_SIZE_EXCEEDED,
-                    "Upload size limit exceeded (max %s bytes)."
-                    % self.config["max_upload_size"],
-                    "Please consider sending the archive in " "multiple steps.",
+                    f"Upload size limit exceeded (max {max_upload_size} bytes)."
+                    "Please consider sending the archive in multiple steps.",
                 )
 
             length = filehandler.size
@@ -341,22 +355,21 @@ class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
                 return make_error_dict(
                     CHECKSUM_MISMATCH,
                     "Wrong md5 hash",
-                    "The checksum sent %s and the actual checksum "
-                    "%s does not match."
-                    % (hashutil.hash_to_hex(md5sum), hashutil.hash_to_hex(_md5sum)),
+                    f"The checksum sent {hashutil.hash_to_hex(md5sum)} and the actual "
+                    f"checksum {hashutil.hash_to_hex(_md5sum)} does not match.",
                 )
 
         return None
 
     def _binary_upload(
         self,
-        request,
-        headers,
-        collection_name,
-        deposit_id=None,
-        replace_metadata=False,
-        replace_archives=False,
-    ):
+        request: Request,
+        headers: Dict[str, Any],
+        collection_name: str,
+        deposit_id: Optional[int] = None,
+        replace_metadata: bool = False,
+        replace_archives: bool = False,
+    ) -> Dict[str, Any]:
         """Binary upload routine.
 
         Other than such a request, a 415 response is returned.
@@ -398,7 +411,7 @@ class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
             return make_error_dict(
                 BAD_REQUEST,
                 "CONTENT_LENGTH header is mandatory",
-                "For archive deposit, the " "CONTENT_LENGTH header must be sent.",
+                "For archive deposit, the CONTENT_LENGTH header must be sent.",
             )
 
         content_disposition = headers["content-disposition"]
@@ -406,15 +419,15 @@ class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
             return make_error_dict(
                 BAD_REQUEST,
                 "CONTENT_DISPOSITION header is mandatory",
-                "For archive deposit, the " "CONTENT_DISPOSITION header must be sent.",
+                "For archive deposit, the CONTENT_DISPOSITION header must be sent.",
             )
 
         packaging = headers["packaging"]
         if packaging and packaging not in ACCEPT_PACKAGINGS:
             return make_error_dict(
                 BAD_REQUEST,
-                "Only packaging %s is supported" % ACCEPT_PACKAGINGS,
-                "The packaging provided %s is not supported" % packaging,
+                f"Only packaging {ACCEPT_PACKAGINGS} is supported",
+                f"The packaging provided {packaging} is not supported",
             )
 
         filehandler = request.FILES["file"]
@@ -450,7 +463,7 @@ class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
             "archive": filehandler.name,
         }
 
-    def _read_metadata(self, metadata_stream):
+    def _read_metadata(self, metadata_stream) -> Tuple[bytes, Dict[str, Any]]:
         """Given a metadata stream, reads the metadata and returns both the
            parsed and the raw metadata.
 
@@ -461,13 +474,13 @@ class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
 
     def _multipart_upload(
         self,
-        request,
-        headers,
-        collection_name,
-        deposit_id=None,
-        replace_metadata=False,
-        replace_archives=False,
-    ):
+        request: Request,
+        headers: Dict[str, Any],
+        collection_name: str,
+        deposit_id: Optional[int] = None,
+        replace_metadata: bool = False,
+        replace_archives: bool = False,
+    ) -> Dict:
         """Multipart upload supported with exactly:
         - 1 archive (zip)
         - 1 atom entry
@@ -477,13 +490,13 @@ class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
         Args:
             request (Request): the request holding information to parse
                 and inject in db
-            headers (dict): request headers formatted
-            collection_name (str): the associated client
-            deposit_id (id): deposit identifier if provided
-            replace_metadata (bool): 'Update or add' request to existing
+            headers: request headers formatted
+            collection_name: the associated client
+            deposit_id: deposit identifier if provided
+            replace_metadata: 'Update or add' request to existing
               deposit. If False (default), this adds new metadata request to
               existing ones. Otherwise, this will replace existing metadata.
-            replace_archives (bool): 'Update or add' request to existing
+            replace_archives: 'Update or add' request to existing
               deposit. If False (default), this adds new archive request to
               existing ones. Otherwise, this will replace existing archives.
               ones.
@@ -510,14 +523,15 @@ class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
 
         content_types_present = set()
 
-        data = {
+        data: Dict[str, Optional[Any]] = {
             "application/zip": None,  # expected either zip
             "application/x-tar": None,  # or x-tar
             "application/atom+xml": None,
         }
         for key, value in request.FILES.items():
             fh = value
-            if fh.content_type in content_types_present:
+            content_type = fh.content_type
+            if content_type in content_types_present:
                 return make_error_dict(
                     ERROR_CONTENT,
                     "Only 1 application/zip (or application/x-tar) archive "
@@ -528,8 +542,9 @@ class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
                     "header in the multipart deposit",
                 )
 
-            content_types_present.add(fh.content_type)
-            data[fh.content_type] = fh
+            content_types_present.add(content_type)
+            assert content_type is not None
+            data[content_type] = fh
 
         if len(content_types_present) != 2:
             return make_error_dict(
@@ -579,6 +594,7 @@ class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
             deposit, deposit_request_data, replace_metadata, replace_archives
         )
 
+        assert filehandler is not None
         return {
             "deposit_id": deposit.id,
             "deposit_date": deposit.reception_date,
@@ -588,25 +604,25 @@ class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
 
     def _atom_entry(
         self,
-        request,
-        headers,
-        collection_name,
-        deposit_id=None,
-        replace_metadata=False,
-        replace_archives=False,
-    ):
+        request: Request,
+        headers: Dict[str, Any],
+        collection_name: str,
+        deposit_id: Optional[int] = None,
+        replace_metadata: bool = False,
+        replace_archives: bool = False,
+    ) -> Dict[str, Any]:
         """Atom entry deposit.
 
         Args:
-            request (Request): the request holding information to parse
+            request: the request holding information to parse
                 and inject in db
-            headers (dict): request headers formatted
-            collection_name (str): the associated client
-            deposit_id (id): deposit identifier if provided
-            replace_metadata (bool): 'Update or add' request to existing
+            headers: request headers formatted
+            collection_name: the associated client
+            deposit_id: deposit identifier if provided
+            replace_metadata: 'Update or add' request to existing
               deposit. If False (default), this adds new metadata request to
               existing ones. Otherwise, this will replace existing metadata.
-            replace_archives (bool): 'Update or add' request to existing
+            replace_archives: 'Update or add' request to existing
               deposit. If False (default), this adds new archive request to
               existing ones. Otherwise, this will replace existing archives.
               ones.
@@ -647,6 +663,10 @@ class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
 
         external_id = metadata.get("external_identifier", headers["slug"])
 
+        # TODO: Determine if we are in the metadata-only deposit case. If it is, then
+        # save deposit and deposit request typed 'metadata' and send metadata to the
+        # metadata storage. Otherwise, do as existing deposit.
+
         deposit = self._deposit_put(
             request,
             deposit_id=deposit_id,
@@ -668,15 +688,17 @@ class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
             "status": deposit.status,
         }
 
-    def _empty_post(self, request, headers, collection_name, deposit_id):
+    def _empty_post(
+        self, request: Request, headers: Dict, collection_name: str, deposit_id: int
+    ) -> Dict[str, Any]:
         """Empty post to finalize an empty deposit.
 
         Args:
-            request (Request): the request holding information to parse
+            request: the request holding information to parse
                 and inject in db
-            headers (dict): request headers formatted
-            collection_name (str): the associated client
-            deposit_id (id): deposit identifier
+            headers: request headers formatted
+            collection_name: the associated client
+            deposit_id: deposit identifier
 
         Returns:
             Dictionary of result with the deposit's id, the date
@@ -695,7 +717,9 @@ class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
             "archive": None,
         }
 
-    def _make_iris(self, request, collection_name, deposit_id):
+    def _make_iris(
+        self, request: Request, collection_name: str, deposit_id: int
+    ) -> Dict[str, Any]:
         """Define the IRI endpoints
 
         Args:
@@ -713,7 +737,13 @@ class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
             for iri in [EM_IRI, EDIT_SE_IRI, CONT_FILE_IRI, STATE_IRI]
         }
 
-    def additional_checks(self, request, headers, collection_name, deposit_id=None):
+    def additional_checks(
+        self,
+        request: Request,
+        headers: Dict[str, Any],
+        collection_name: str,
+        deposit_id: Optional[int] = None,
+    ) -> Dict[str, Any]:
         """Permit the child class to enrich additional checks.
 
         Returns:
@@ -722,41 +752,50 @@ class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
         """
         return {}
 
-    def checks(self, request, collection_name, deposit_id=None):
+    def checks(
+        self, request: Request, collection_name: str, deposit_id: Optional[int] = None
+    ) -> Dict[str, Any]:
         try:
             self._collection = DepositCollection.objects.get(name=collection_name)
         except DepositCollection.DoesNotExist:
             return make_error_dict(
-                NOT_FOUND, "Unknown collection name %s" % collection_name
+                NOT_FOUND, f"Unknown collection name {collection_name}"
             )
+        assert self._collection is not None
 
         username = request.user.username
         if username:  # unauthenticated request can have the username empty
             try:
-                self._client = DepositClient.objects.get(username=username)
+                self._client: DepositClient = DepositClient.objects.get(  # type: ignore
+                    username=username
+                )
             except DepositClient.DoesNotExist:
-                return make_error_dict(NOT_FOUND, "Unknown client name %s" % username)
+                return make_error_dict(NOT_FOUND, f"Unknown client name {username}")
 
-            if self._collection.id not in self._client.collections:
+            collection_id = self._collection.id
+            collections = self._client.collections
+            assert collections is not None
+            if collection_id not in collections:
                 return make_error_dict(
                     FORBIDDEN,
-                    "Client %s cannot access collection %s"
-                    % (username, collection_name),
+                    f"Client {username} cannot access collection {collection_name}",
                 )
 
+        headers = self._read_headers(request)
+
         if deposit_id:
             try:
                 deposit = Deposit.objects.get(pk=deposit_id)
             except Deposit.DoesNotExist:
                 return make_error_dict(
-                    NOT_FOUND, "Deposit with id %s does not exist" % deposit_id
+                    NOT_FOUND, f"Deposit with id {deposit_id} does not exist"
                 )
 
-            checks = self.restrict_access(request, deposit)
+            assert deposit is not None
+            checks = self.restrict_access(request, headers, deposit)
             if checks:
                 return checks
 
-        headers = self._read_headers(request)
         if headers["on-behalf-of"]:
             return make_error_dict(MEDIATION_NOT_ALLOWED, "Mediation is not supported.")
 
@@ -766,43 +805,58 @@ class SWHBaseDeposit(SWHDefaultConfig, SWHAPIView, metaclass=ABCMeta):
 
         return {"headers": headers}
 
-    def restrict_access(self, request, deposit=None):
-        if deposit:
-            if request.method != "GET" and deposit.status != DEPOSIT_STATUS_PARTIAL:
-                summary = "You can only act on deposit with status '%s'" % (
-                    DEPOSIT_STATUS_PARTIAL,
-                )
-                description = "This deposit has status '%s'" % deposit.status
-                return make_error_dict(
-                    BAD_REQUEST, summary=summary, verbose_description=description
-                )
+    def restrict_access(
+        self, request: Request, headers: Dict, deposit: Deposit
+    ) -> Dict[str, Any]:
+        """Allow modifications on deposit with status 'partial' only, reject the rest.
 
-    def _basic_not_allowed_method(self, request, method):
+        """
+        if request.method != "GET" and deposit.status != DEPOSIT_STATUS_PARTIAL:
+            summary = "You can only act on deposit with status '%s'" % (
+                DEPOSIT_STATUS_PARTIAL,
+            )
+            description = f"This deposit has status '{deposit.status}'"
+            return make_error_dict(
+                BAD_REQUEST, summary=summary, verbose_description=description
+            )
+        return {}
+
+    def _basic_not_allowed_method(self, request: Request, method: str):
         return make_error_response(
             request,
             METHOD_NOT_ALLOWED,
-            "%s method is not supported on this endpoint" % method,
+            f"{method} method is not supported on this endpoint",
         )
 
-    def get(self, request, *args, **kwargs):
+    def get(
+        self, request: Request, collection_name: str, deposit_id: int
+    ) -> Union[HttpResponse, FileResponse]:
         return self._basic_not_allowed_method(request, "GET")
 
-    def post(self, request, *args, **kwargs):
+    def post(
+        self, request: Request, collection_name: str, deposit_id: Optional[int] = None
+    ) -> HttpResponse:
         return self._basic_not_allowed_method(request, "POST")
 
-    def put(self, request, *args, **kwargs):
+    def put(
+        self, request: Request, collection_name: str, deposit_id: int
+    ) -> HttpResponse:
         return self._basic_not_allowed_method(request, "PUT")
 
-    def delete(self, request, *args, **kwargs):
+    def delete(
+        self, request: Request, collection_name: str, deposit_id: Optional[int] = None
+    ) -> HttpResponse:
         return self._basic_not_allowed_method(request, "DELETE")
 
 
-class SWHGetDepositAPI(SWHBaseDeposit, metaclass=ABCMeta):
+class APIGet(APIBase, metaclass=ABCMeta):
     """Mixin for class to support GET method.
 
     """
 
-    def get(self, request, collection_name, deposit_id, format=None):
+    def get(
+        self, request: Request, collection_name: str, deposit_id: int
+    ) -> Union[HttpResponse, FileResponse]:
         """Endpoint to create/add resources to deposit.
 
         Returns:
@@ -817,14 +871,22 @@ class SWHGetDepositAPI(SWHBaseDeposit, metaclass=ABCMeta):
 
         r = self.process_get(request, collection_name, deposit_id)
 
-        if isinstance(r, tuple):
-            status, content, content_type = r
-            return HttpResponse(content, status=status, content_type=content_type)
-
-        return r
+        status, content, content_type = r
+        if content_type == "swh/generator":
+            with content as path:
+                return FileResponse(
+                    open(path, "rb"), status=status, content_type="application/zip"
+                )
+        if content_type == "application/json":
+            return HttpResponse(
+                json.dumps(content), status=status, content_type=content_type
+            )
+        return HttpResponse(content, status=status, content_type=content_type)
 
     @abstractmethod
-    def process_get(self, request, collection_name, deposit_id):
+    def process_get(
+        self, request: Request, collection_name: str, deposit_id: int
+    ) -> Tuple[int, Any, str]:
         """Routine to deal with the deposit's get processing.
 
         Returns:
@@ -834,12 +896,14 @@ class SWHGetDepositAPI(SWHBaseDeposit, metaclass=ABCMeta):
         pass
 
 
-class SWHPostDepositAPI(SWHBaseDeposit, metaclass=ABCMeta):
+class APIPost(APIBase, metaclass=ABCMeta):
     """Mixin for class to support DELETE method.
 
     """
 
-    def post(self, request, collection_name, deposit_id=None, format=None):
+    def post(
+        self, request: Request, collection_name: str, deposit_id: Optional[int] = None
+    ) -> HttpResponse:
         """Endpoint to create/add resources to deposit.
 
         Returns:
@@ -871,11 +935,17 @@ class SWHPostDepositAPI(SWHBaseDeposit, metaclass=ABCMeta):
             content_type="application/xml",
             status=_status,
         )
-        response._headers["location"] = "Location", data[_iri_key]
+        response._headers["location"] = "Location", data[_iri_key]  # type: ignore
         return response
 
     @abstractmethod
-    def process_post(self, request, headers, collection_name, deposit_id=None):
+    def process_post(
+        self,
+        request,
+        headers: Dict,
+        collection_name: str,
+        deposit_id: Optional[int] = None,
+    ) -> Tuple[int, str, Dict]:
         """Routine to deal with the deposit's processing.
 
         Returns
@@ -888,12 +958,14 @@ class SWHPostDepositAPI(SWHBaseDeposit, metaclass=ABCMeta):
         pass
 
 
-class SWHPutDepositAPI(SWHBaseDeposit, metaclass=ABCMeta):
+class APIPut(APIBase, metaclass=ABCMeta):
     """Mixin for class to support PUT method.
 
     """
 
-    def put(self, request, collection_name, deposit_id, format=None):
+    def put(
+        self, request: Request, collection_name: str, deposit_id: int
+    ) -> HttpResponse:
         """Endpoint to update deposit resources.
 
         Returns:
@@ -916,7 +988,9 @@ class SWHPutDepositAPI(SWHBaseDeposit, metaclass=ABCMeta):
         return HttpResponse(status=status.HTTP_204_NO_CONTENT)
 
     @abstractmethod
-    def process_put(self, request, headers, collection_name, deposit_id):
+    def process_put(
+        self, request: Request, headers: Dict, collection_name: str, deposit_id: int
+    ) -> Dict[str, Any]:
         """Routine to deal with updating a deposit in some way.
 
         Returns
@@ -926,12 +1000,14 @@ class SWHPutDepositAPI(SWHBaseDeposit, metaclass=ABCMeta):
         pass
 
 
-class SWHDeleteDepositAPI(SWHBaseDeposit, metaclass=ABCMeta):
+class APIDelete(APIBase, metaclass=ABCMeta):
     """Mixin for class to support DELETE method.
 
     """
 
-    def delete(self, request, collection_name, deposit_id):
+    def delete(
+        self, request: Request, collection_name: str, deposit_id: Optional[int] = None
+    ) -> HttpResponse:
         """Endpoint to delete some deposit's resources (archives, deposit).
 
         Returns:
@@ -944,6 +1020,7 @@ class SWHDeleteDepositAPI(SWHBaseDeposit, metaclass=ABCMeta):
         if "error" in checks:
             return make_error_response_from_dict(request, checks["error"])
 
+        assert deposit_id is not None
         data = self.process_delete(request, collection_name, deposit_id)
         error = data.get("error")
         if error:
@@ -952,11 +1029,13 @@ class SWHDeleteDepositAPI(SWHBaseDeposit, metaclass=ABCMeta):
         return HttpResponse(status=status.HTTP_204_NO_CONTENT)
 
     @abstractmethod
-    def process_delete(self, request, collection_name, deposit_id):
+    def process_delete(
+        self, request: Request, collection_name: str, deposit_id: int
+    ) -> Dict:
         """Routine to delete a resource.
 
         This is mostly not allowed except for the
-        EM_IRI (cf. .api.deposit_update.SWHUpdateArchiveDeposit)
+        EM_IRI (cf. .api.deposit_update.APIUpdateArchive)
 
         """
-        pass
+        return {}
